export { isDef, isSrc } from './validate'
/**
 * 获取链接某个参数 search
 * @param {String} name 参数名称
 * @returns {String} 返回参数值
 */
export function getQueryString(name) {
  let reg = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i')
  let r = window.location.search.substr(1).match(reg)
  // eslint-disable-next-line eqeqeq, no-eq-null
  if (r != null) {
    return decodeURIComponent(r[2])
  }
  return null
}

/**
 * 获取网址参数 search 和 hash
 * @returns {Object} 返回包含当前URL参数的对象
 *
 */
export function getURLParameters() {
  const reg = /([^?=&]+)(=([^&]*))/g
  const searchParamList = window.location.search.match(reg)
  const hashParamList = window.location.hash.match(reg)
  const obj = {}

  searchParamList &&
    searchParamList.forEach((v) => {
      obj[v.slice(0, v.indexOf('='))] = decodeURIComponent(v.slice(v.indexOf('=') + 1))
    })

  hashParamList &&
    hashParamList.forEach((v) => {
      obj[v.slice(0, v.indexOf('='))] = decodeURIComponent(v.slice(v.indexOf('=') + 1))
    })

  return obj
}
/**
 * getEnv
 * @returns {} 环境信息
 */
export function getEnv() {
  const ua = navigator.userAgent.toLowerCase() // 一律小写
  const isAndroid = /android/i.test(ua)
  const isIOS = /iphone|ipad|ipod|ios/i.test(ua)
  const isInMiniProgram = /miniProgram/i.test(ua) // 微信小程序内
  const isInWeixinApp = /micromessenger/i.test(ua) // 微信内
  const isInWeChatDevTools = /wechatdevtools/i.test(ua) // 微信开发者工具内

  return {
    isAndroid,
    isIOS,
    isInMiniProgram,
    isInWeixinApp,
    isInWeChatDevTools,
  }
}

/**
 * 接口前缀替换为当前部署站点域名
 * @param {*} url
 * @param {*} host
 * @return {String}
 */
export function replaceHost(str, host = process.env.VUE_APP_BASE_HOST) {
  let url = str
  if (process.env.NODE_ENV === 'production') {
    const patten = new RegExp(host)
    url = str.replace(patten, location.host)
  }

  return url
}
/**
 *
 * @param {*} code api, host, origin
 * @returns
 */
export function getAPI(code = 'api') {
  const api = replaceHost(process.env.VUE_APP_BASE_API)

  if (code === 'origin') {
    return replaceHost(process.env.VUE_APP_BASE_ORIGIN)
  }

  if (code === 'host') {
    return replaceHost(process.env.VUE_APP_BASE_HOST)
  }

  return api
}
/**
 * idDefString
 * @param {*} str
 */
export function idDefString(str) {
  return str !== 'undefined' && str !== 'null'
}
/**
 * 获取服务之窗的缓存
 * @param {*} key
 * @param {*} def
 * @returns
 */
export function getLocalforage(key, def) {
  const str = localStorage.getItem(`localforage/${key}`)
  if (str) {
    try {
      return idDefString(JSON.parse(str)) ? JSON.parse(str) : ''
    } catch (e) {
      return str || null
    }
  } else {
    // eslint-disable-next-line eqeqeq
    return def != undefined ? def : null
  }
}

/**
 * 设置服务之窗的缓存
 * @param {*} key
 * @param {*} data
 * @returns
 */
export function setLocalforage(key, data) {
  localStorage.setItem(`localforage/${key}`, JSON.stringify(data))
}

/**
 * 生成版本号（时间戳 按5分钟）
 * @returns {String}
 */
export function getTimeStampVersion() {
  const now = new Date()
  const year = now.getFullYear()
  const month = now.getMonth() + 1
  const date = now.getDate()
  const hour = now.getHours()
  const minutes = now.getMinutes()
  const padZero = (n) => String(n).padStart(2, '0')
  const base = 5

  return `${year}${padZero(month)}${padZero(date)}${padZero(hour)}${padZero(base * Math.floor(minutes / base))}`
}

/**
 * 给url 追加参数
 * @param {*} url
 * @param {Object} query
 * @description 多加了个版本号_v，避免缓存
 * @returns {String}
 */
export function interceptorUrl(url, query = {}) {
  const search = url.split('#')[0]
  const hash = url.split('#')[1]
  const params = {
    ...query,
    _v: getTimeStampVersion(),
  }
  // eslint-disable-next-line no-implicit-coercion
  let link = `${search}${~search.indexOf('?') ? '&' : '?'}`

  link += Object.keys(params)
    .map((key) => key + '=' + encodeURIComponent(params[key]))
    .join('&')

  if (hash) {
    link += `#${hash}`
  }

  return link
}

/**
 * 验证 - 是否是手机号
 * @param {String} val
 */
export function isPhone(value) {
  const reg = /^1[0-9]{10}$/
  return reg.test(value)
}
