/**
 *Lodash 通过降低 array、number、objects、string 等等的使用难度从而让 JavaScript 变得更简单。
 */

/**
 * 函数节流
 * @param {*} fn 事件回调
 * @param {*} interval 时间间隔的阈值
 */
export const throttle = function(fn, interval) {
  let last = 0
  return function() {
    // eslint-disable-next-line no-invalid-this
    const context = this
    const args = arguments
    const now = Number(new Date())

    if (now - last >= interval) {
      last = now
      fn.apply(context, args)
    }
  }
}

/**
 * 函数防抖
 * @param {callback} fn 事件回调
 * @param {number} delay 每次推迟执行的等待时间
 */
export const debounce = function(fn, delay) {
  let last = 0
  let timer = null
  return function() {
    // eslint-disable-next-line no-invalid-this
    const context = this
    const args = arguments
    const now = Number(new Date())

    if (now - last < delay) {
      clearTimeout(timer)
      timer = setTimeout(function() {
        last = now
        fn.apply(context, args)
      }, delay)
    } else {
      last = now
      fn.apply(context, args)
    }
  }
}

/**
 * 浅拷贝
 * @param {Object} source
 */
export function shallowClone(source) {
  if (!source && typeof source !== 'object') {
    throw new Error('error arguments', 'shallowClone')
  }
  return { ...source }
}

/**
 * 深拷贝
 * @param {Object} source
 */
export function deepClone(source) {
  if (!source && typeof source !== 'object') {
    throw new Error('error arguments', 'deepClone')
  }
  const targetObj = source.constructor === Array ? [] : {}
  Object.keys(source).forEach((keys) => {
    if (source[keys] && typeof source[keys] === 'object') {
      targetObj[keys] = deepClone(source[keys])
    } else {
      targetObj[keys] = source[keys]
    }
  })
  return targetObj
}

/**
 * 生成一个 UUID
 */
export const guid = function() {
  return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
    let r = (Math.random() * 16) | 0
    let v = c === 'x' ? r : (r & 0x3) | 0x8
    return v.toString(16)
  })
}

/**
 * 数组转map
 * @param {Array} array
 * @param {String} array
 */
export const arrayToMap = function(array, key) {
  const map = {}
  if (array.length) {
    array.forEach((item) => {
      map[item[key]] = item
    })
  }
  return map
}
